@@ -34,6 +34,8 @@ class _RayRunnerConfig(_RunnerConfig):
 class _DynamicRayRunnerConfig(_RunnerConfig):
     name = "dynamicray"
     address: str | None
+    max_tasks_per_core: int | None
+    max_refs_per_core: int | None
 
 
 def _get_runner_config_from_env() -> _RunnerConfig:
@@ -53,7 +55,13 @@ def _get_runner_config_from_env() -> _RunnerConfig:
         elif runner.upper() == "DYNAMIC":
             return _DynamicRunnerConfig()
         elif runner.upper() == "DYNAMICRAY":
-            return _DynamicRayRunnerConfig(address=os.getenv("DAFT_RAY_ADDRESS"))
+            tasks_per_core_env = os.getenv("DAFT_RAY_MAX_TASKS_PER_CORE")
+            refs_per_core_env = os.getenv("DAFT_RAY_MAX_REFS_PER_CORE")
+            return _DynamicRayRunnerConfig(
+                address=os.getenv("DAFT_RAY_ADDRESS"),
+                max_tasks_per_core=int(tasks_per_core_env) if tasks_per_core_env else None,
+                max_refs_per_core=int(refs_per_core_env) if refs_per_core_env else None,
+            )
         raise ValueError(f"Unsupported DAFT_RUNNER variable: {os.environ['DAFT_RUNNER']}")
     return _PyRunnerConfig()
 
@@ -96,7 +104,11 @@ class DaftContext:
 
             logger.info("Using DynamicRayRunner")
             assert isinstance(self.runner_config, _DynamicRayRunnerConfig)
-            _RUNNER = DynamicRayRunner(address=self.runner_config.address)
+            _RUNNER = DynamicRayRunner(
+                address=self.runner_config.address,
+                max_tasks_per_core=self.runner_config.max_tasks_per_core,
+                max_refs_per_core=self.runner_config.max_refs_per_core,
+            )
         else:
             raise NotImplementedError(f"Runner config implemented: {self.runner_config.name}")
 
@@ -143,14 +155,20 @@ def set_runner_ray(address: str | None = None) -> DaftContext:
     return _DaftContext
 
 
-def set_runner_dynamic_ray(address: str | None = None) -> DaftContext:
+def set_runner_dynamic_ray(
+    address: str | None = None,
+    max_tasks_per_core: int | None = None,
+    max_refs_per_core: int | None = None,
+) -> DaftContext:
     """[Experimental] Sets the runner for executing Daft dataframes to the DynamicRayRunner."""
     global _DaftContext
     if _DaftContext.disallow_set_runner:
         raise RuntimeError("Cannot set runner more than once")
     _DaftContext = dataclasses.replace(
         _DaftContext,
-        runner_config=_DynamicRayRunnerConfig(address=address),
+        runner_config=_DynamicRayRunnerConfig(
+            address=address, max_tasks_per_core=max_tasks_per_core, max_refs_per_core=max_refs_per_core
+        ),
         disallow_set_runner=True,
     )
     return _DaftContext
