@@ -30,12 +30,15 @@ from daft.execution.execution_step import (
 from daft.logical import logical_plan
 
 PartitionT = TypeVar("PartitionT")
-_PartitionT = TypeVar("_PartitionT")
+T = TypeVar("T")
 
 
 def partition_read(partitions: Iterator[PartitionT]) -> Iterator[None | ExecutionStep[PartitionT]]:
     """Instantiate a (no-op) physical plan from existing partitions."""
-    yield from (OpenExecutionQueue[PartitionT](inputs=[partition], instructions=[]) for partition in partitions)
+    yield from (
+        OpenExecutionQueue[PartitionT](inputs=[partition], instructions=[], resource_request=None)
+        for partition in partitions
+    )
 
 
 def file_read(
@@ -47,7 +50,10 @@ def file_read(
     Yield a plan to read those filenames.
     """
     yield from (
-        step.add_instruction(execution_step.ReadFile(partition_id=index, logplan=scan_info))
+        step.add_instruction(
+            execution_step.ReadFile(partition_id=index, logplan=scan_info),
+            resource_request=None,
+        )
         if isinstance(step, OpenExecutionQueue)
         else step
         for index, step in enumerate_open_executions(child_plan)
@@ -61,7 +67,10 @@ def file_write(
     """Write the results of `child_plan` into files described by `write_info`."""
 
     yield from (
-        step.add_instruction(execution_step.WriteFile(partition_id=index, logplan=write_info))
+        step.add_instruction(
+            execution_step.WriteFile(partition_id=index, logplan=write_info),
+            resource_request=None,
+        )
         if isinstance(step, OpenExecutionQueue)
         else step
         for index, step in enumerate_open_executions(child_plan)
@@ -71,11 +80,12 @@ def file_write(
 def pipeline_instruction(
     child_plan: Iterator[None | ExecutionStep[PartitionT]],
     pipeable_instruction: Instruction,
+    resource_request: execution_step.ResourceRequest | None,
 ) -> Iterator[None | ExecutionStep[PartitionT]]:
     """Apply an instruction to the results of `child_plan`."""
 
     yield from (
-        step.add_instruction(pipeable_instruction) if isinstance(step, OpenExecutionQueue) else step
+        step.add_instruction(pipeable_instruction, resource_request) if isinstance(step, OpenExecutionQueue) else step
         for step in child_plan
     )
 
@@ -108,6 +118,7 @@ def join(
             join_step = OpenExecutionQueue[PartitionT](
                 inputs=[next_left.result.partition(), next_right.result.partition()],
                 instructions=[execution_step.Join(join)],
+                resource_request=None,
             )
             yield join_step
 
@@ -139,21 +150,25 @@ def join(
 def local_limit(
     child_plan: Iterator[None | ExecutionStep[PartitionT]],
     limit: int,
-) -> Generator[None | ExecutionStep[PartitionT], int, None]:
+) -> Generator[None | ExecutionStep[PartitionT], None | int, None]:
     """Apply a limit instruction to each partition in the child_plan.
 
     limit:
-        The value of the limit to apply to the first partition.
-        For subsequent partitions, send the value of the limit to apply back into this generator.
+        The value of the limit to apply to each partition.
 
     Yields: ExecutionStep with the limit applied.
-    Send back: The next limit to apply.
+    Send back: A new value to the limit (optional). This allows you to update the limit after each partition if desired.
     """
     for step in child_plan:
         if not isinstance(step, OpenExecutionQueue):
             yield step
         else:
-            limit = yield step.add_instruction(execution_step.LocalLimit(limit))
+            maybe_new_limit = yield step.add_instruction(
+                execution_step.LocalLimit(limit),
+                resource_request=None,
+            )
+            if maybe_new_limit is not None:
+                limit = maybe_new_limit
 
 
 def global_limit(
@@ -163,6 +178,7 @@ def global_limit(
     """Return the first n rows from the `child_plan`."""
 
     remaining_rows = global_limit._num
+    assert remaining_rows >= 0, f"Invalid value for limit: {remaining_rows}"
     remaining_partitions = global_limit.num_partitions()
 
     materializations: deque[MaterializationRequest[PartitionT]] = deque()
@@ -188,6 +204,7 @@ def global_limit(
             global_limit_step = OpenExecutionQueue[PartitionT](
                 inputs=[result.partition()],
                 instructions=[execution_step.LocalLimit(limit)],
+                resource_request=None,
             )
             yield global_limit_step
             remaining_partitions -= 1
@@ -208,6 +225,7 @@ def global_limit(
                     OpenExecutionQueue[PartitionT](
                         inputs=[result.partition()],
                         instructions=[execution_step.LocalLimit(0)],
+                        resource_request=None,
                     )
                     for _ in range(remaining_partitions)
                 )
@@ -260,13 +278,16 @@ def coalesce(
         num_partitions_to_merge = merges_per_result[0]
         if len(materializations) >= num_partitions_to_merge:
             ready_to_coalesce = [
-                result for i in range(num_partitions_to_merge) if (result := materializations[i].result) is not None
+                materializations[i].result
+                for i in range(num_partitions_to_merge)
+                if materializations[i].result is not None
             ]
             if len(ready_to_coalesce) == num_partitions_to_merge:
                 # Coalesce the partition and emit it.
                 merge_step = OpenExecutionQueue[PartitionT](
                     inputs=[_.partition() for _ in ready_to_coalesce],
                     instructions=[execution_step.ReduceMerge()],
+                    resource_request=None,
                 )
                 [materializations.popleft() for _ in range(num_partitions_to_merge)]
                 merges_per_result.popleft()
@@ -314,16 +335,17 @@ def reduce(
     # (since we need all of them to emit even a single reduce).
     while any(_.results is None for _ in materializations):
         yield None
-    assert materializations[0].results is not None  # for mypy only
+
+    inputs_to_reduce = [deque(_.results) for _ in materializations if _.results is not None]
+    del materializations
 
     # Yield all the reduces in order.
-    yield from (
-        OpenExecutionQueue[PartitionT](
-            inputs=[result.partition() for result in (_.results[reduce_index] for _ in materializations)],
+    while len(inputs_to_reduce[0]) > 0:
+        yield OpenExecutionQueue[PartitionT](
+            inputs=[result.partition() for result in (_.popleft() for _ in inputs_to_reduce)],
             instructions=[reduce_instruction],
+            resource_request=None,
         )
-        for reduce_index in range(len(materializations[0].results))
-    )
 
 
 def sort(
@@ -333,7 +355,7 @@ def sort(
     """Sort the result of `child_plan` according to `sort_info`."""
 
     # First, materialize the child plan.
-    source_materializations: list[MaterializationRequest[PartitionT]] = list()
+    source_materializations: deque[MaterializationRequest[PartitionT]] = deque()
     for step in child_plan:
         if isinstance(step, OpenExecutionQueue):
             step = step.as_materialization_request()
@@ -341,13 +363,14 @@ def sort(
         yield step
 
     # Sample all partitions (to be used for calculating sort boundaries).
-    sample_materializations: list[MaterializationRequest[PartitionT]] = list()
+    sample_materializations: deque[MaterializationRequest[PartitionT]] = deque()
     for source in source_materializations:
         while source.result is None:
             yield None
         sample = OpenExecutionQueue[PartitionT](
             inputs=[source.result.partition()],
             instructions=[execution_step.Sample(sort_by=sort_info._sort_by)],
+            resource_request=None,
         ).as_materialization_request()
         sample_materializations.append(sample)
         yield sample
@@ -357,13 +380,12 @@ def sort(
         yield None
 
     # Reduce the samples to get sort boundaries.
-    samples = [
-        sample.result.partition()
-        for sample in sample_materializations
-        if sample.result is not None  # for mypy only; guaranteed to be not None by while loop
-    ]
     boundaries = OpenExecutionQueue[PartitionT](
-        inputs=samples,
+        inputs=[
+            sample.result.partition()
+            for sample in consume_deque(sample_materializations)
+            if sample.result is not None  # for mypy only; guaranteed to be not None by while loop
+        ],
         instructions=[
             execution_step.ReduceToQuantiles(
                 num_quantiles=sort_info.num_partitions(),
@@ -371,6 +393,7 @@ def sort(
                 descending=sort_info._descending,
             ),
         ],
+        resource_request=None,
     ).as_materialization_request()
     yield boundaries
 
@@ -381,7 +404,6 @@ def sort(
     boundaries_partition = boundaries.result.partition()
 
     # Create a range fanout plan.
-    source_partitions = [source.result.partition() for source in source_materializations if source.result is not None]
     range_fanout_plan = (
         OpenExecutionQueue[PartitionT](
             inputs=[boundaries_partition, source_partition],
@@ -392,8 +414,11 @@ def sort(
                     descending=sort_info._descending,
                 ),
             ],
+            resource_request=None,
+        )
+        for source_partition in (
+            source.result.partition() for source in consume_deque(source_materializations) if source.result is not None
         )
-        for source_partition in source_partitions
     )
 
     # Execute a sorting reduce on it.
@@ -439,9 +464,14 @@ def enumerate_open_executions(
     Intended for counting the number of OpenExecutionQueues returned by the iterator.
     """
     index = 0
-    yield from (
-        (((index := index + 1) - 1), item)  # aka (index++, item)
-        if isinstance(item, OpenExecutionQueue)
-        else (index, item)
-        for item in schedule
-    )
+    for item in schedule:
+        if isinstance(item, OpenExecutionQueue):
+            yield index, item
+            index += 1
+        else:
+            yield index, item
+
+
+def consume_deque(dq: deque[T]) -> Iterator[T]:
+    while len(dq) > 0:
+        yield dq.popleft()
