@@ -1,8 +1,11 @@
 from __future__ import annotations
 
+import copy
+
 from loguru import logger
 
-from daft.expressions import ColID, ColumnExpression
+from daft import resource_request
+from daft.expressions import col
 from daft.internal.rule import Rule
 from daft.logical.logical_plan import (
     Coalesce,
@@ -34,14 +37,17 @@ class PushDownPredicates(Rule[LogicalPlan]):
 
     def _filter_through_projection(self, parent: Filter, child: Projection) -> LogicalPlan | None:
         filter_predicate = parent._predicate
-
         grandchild = child._children()[0]
-        ids_produced_by_grandchild = grandchild.schema().to_id_set()
+        child_input_mapping = child._projection.input_mapping()
+
         can_push_down = []
         can_not_push_down = []
         for pred in filter_predicate:
-            id_set = {e.get_id() for e in pred.required_columns()}
-            if id_set.issubset(ids_produced_by_grandchild):
+            required_names = {e for e in pred.required_columns()}
+            if all(name in child_input_mapping for name in required_names):
+                pred = copy.deepcopy(pred)
+                for name in required_names:
+                    pred = pred._replace_column_with_expression(col(name), col(child_input_mapping[name]))
                 can_push_down.append(pred)
             else:
                 can_not_push_down.append(pred)
@@ -49,7 +55,9 @@ class PushDownPredicates(Rule[LogicalPlan]):
             return None
         logger.debug(f"Pushing down Filter predicate {can_push_down} into {child}")
         pushed_down_filter = Projection(
-            input=Filter(grandchild, predicate=ExpressionList(can_push_down)), projection=child._projection
+            input=Filter(grandchild, predicate=ExpressionList(can_push_down)),
+            projection=child._projection,
+            custom_resource_request=child.resource_request(),
         )
 
         if len(can_not_push_down) == 0:
@@ -66,17 +74,27 @@ class PushDownPredicates(Rule[LogicalPlan]):
     def _filter_through_join(self, parent: Filter, child: Join) -> LogicalPlan | None:
         left = child._children()[0]
         right = child._children()[1]
-        left_ids = left.schema().to_id_set()
-        right_ids = right.schema().to_id_set()
+        left.schema().to_name_set()
+        right.schema().to_name_set()
+
+        left_input_mapping = child._left_columns.input_mapping()
+        right_input_mapping = child._right_columns.input_mapping()
+
         filter_predicate = parent._predicate
         can_not_push_down = []
         left_push_down = []
         right_push_down = []
         for pred in filter_predicate:
-            id_set = {e.get_id() for e in pred.required_columns()}
-            if id_set.issubset(left_ids):
+            required_names = pred.required_columns()
+            if all(name in left_input_mapping for name in required_names):
+                pred = copy.deepcopy(pred)
+                for name in required_names:
+                    pred = pred._replace_column_with_expression(col(name), col(left_input_mapping[name]))
                 left_push_down.append(pred)
-            elif id_set.issubset(right_ids):
+            elif all(name in right_input_mapping for name in required_names):
+                pred = copy.deepcopy(pred)
+                for name in required_names:
+                    pred = pred._replace_column_with_expression(col(name), col(right_input_mapping[name]))
                 right_push_down.append(pred)
             else:
                 can_not_push_down.append(pred)
@@ -113,25 +131,35 @@ class PruneColumns(Rule[LogicalPlan]):
         self.register_fn(LocalAggregate, LogicalPlan, self._aggregate_logical_plan)
 
     def _projection_projection(self, parent: Projection, child: Projection) -> LogicalPlan | None:
-        parent_required_set = parent.required_columns().to_id_set()
-        child_output_set = child.schema().to_id_set()
+        parent_required_set = parent.required_columns()
+        child_output_set = child.schema().to_name_set()
         if child_output_set.issubset(parent_required_set):
             return None
 
         logger.debug(f"Pruning Columns: {child_output_set - parent_required_set} in projection projection")
 
-        new_child_exprs = [e for e in child.schema() if e.get_id() in parent_required_set]
+        child_projections = child._projection
+        new_child_exprs = [e for e in child_projections if e.name() in parent_required_set]
         grandchild = child._children()[0]
-        return parent.copy_with_new_children([Projection(grandchild, projection=ExpressionList(new_child_exprs))])
+
+        return parent.copy_with_new_children(
+            [
+                Projection(
+                    grandchild,
+                    projection=ExpressionList(new_child_exprs),
+                    custom_resource_request=child.resource_request(),
+                )
+            ]
+        )
 
     def _projection_aggregate(self, parent: Projection, child: LocalAggregate) -> LogicalPlan | None:
-        parent_required_set = parent.required_columns().to_id_set()
+        parent_required_set = parent.required_columns()
         agg_op_pairs = child._agg
-        agg_ids = {e.get_id() for e, _ in agg_op_pairs}
+        agg_ids = {e.name() for e, _ in agg_op_pairs}
         if agg_ids.issubset(parent_required_set):
             return None
 
-        new_agg_pairs = [(e, op) for e, op in agg_op_pairs if e.get_id() in parent_required_set]
+        new_agg_pairs = [(e, op) for e, op in agg_op_pairs if e.name() in parent_required_set]
         grandchild = child._children()[0]
 
         logger.debug(f"Pruning Columns:  {agg_ids - parent_required_set} in projection aggregate")
@@ -140,8 +168,8 @@ class PruneColumns(Rule[LogicalPlan]):
         )
 
     def _aggregate_logical_plan(self, parent: LocalAggregate, child: LogicalPlan) -> LogicalPlan | None:
-        parent_required_set = parent.required_columns().to_id_set()
-        child_output_set = child.schema().to_id_set()
+        parent_required_set = parent.required_columns()
+        child_output_set = child.schema().to_name_set()
         if child_output_set.issubset(parent_required_set):
             return None
 
@@ -154,23 +182,22 @@ class PruneColumns(Rule[LogicalPlan]):
             return None
         if len(child._children()) == 0:
             return None
-        required_set = parent.required_columns().to_id_set().union(child.required_columns().to_id_set())
-        child_output_set = child.schema().to_id_set()
+        required_set = parent.required_columns() | child.required_columns()
+        child_output_set = child.schema().to_name_set()
         if child_output_set.issubset(required_set):
             return None
         logger.debug(f"Pruning Columns: {child_output_set - required_set} in projection logical plan")
         new_grandchildren = [self._create_pruning_child(gc, required_set) for gc in child._children()]
         return parent.copy_with_new_children([child.copy_with_new_children(new_grandchildren)])
 
-    def _create_pruning_child(self, child: LogicalPlan, parent_id_set: set[ColID]) -> LogicalPlan:
-        child_ids = child.schema().to_id_set()
-        if child_ids.issubset(parent_id_set):
+    def _create_pruning_child(self, child: LogicalPlan, parent_name_set: set[str]) -> LogicalPlan:
+        child_ids = child.schema().to_name_set()
+        if child_ids.issubset(parent_name_set):
             return child
         return Projection(
             child,
-            projection=ExpressionList(
-                [e.to_column_expression() for e in child.schema() if e.get_id() in parent_id_set]
-            ),
+            projection=ExpressionList([e.to_column_expression() for e in child.schema() if e.name in parent_name_set]),
+            custom_resource_request=None,
         )
 
 
@@ -181,7 +208,7 @@ class CombineFilters(Rule[LogicalPlan]):
 
     def _combine_filters(self, parent: Filter, child: Filter) -> Filter:
         logger.debug(f"combining {parent} into {child}")
-        new_predicate = parent._predicate.union(child._predicate, rename_dup="right.")
+        new_predicate = parent._predicate.union(child._predicate, rename_dup="copyname.")
         grand_child = child._children()[0]
         return Filter(grand_child, new_predicate)
 
@@ -219,20 +246,20 @@ class PushDownClausesIntoScan(Rule[LogicalPlan]):
         self.register_fn(Projection, TabularFilesScan, self._push_down_projections_into_scan)
 
     def _push_down_projections_into_scan(self, parent: Projection, child: TabularFilesScan) -> LogicalPlan | None:
-        required_columns = parent.schema().required_columns()
+        required_columns = parent._projection.required_columns()
         scan_columns = child.schema()
-        if required_columns.to_id_set() == scan_columns.to_id_set():
+        if required_columns == scan_columns.to_name_set():
             return None
 
         new_scan = TabularFilesScan(
             schema=child._schema,
             predicate=child._predicate,
-            columns=required_columns.names,
+            columns=list(required_columns),
             source_info=child._source_info,
             filepaths_child=child._filepaths_child,
             filepaths_column_name=child._filepaths_column_name,
         )
-        if any(not isinstance(e, ColumnExpression) for e in parent._projection):
+        if any(not e.is_column() for e in parent._projection):
             return parent.copy_with_new_children([new_scan])
         else:
             return new_scan
@@ -245,28 +272,36 @@ class FoldProjections(Rule[LogicalPlan]):
 
     def _drop_double_projection(self, parent: Projection, child: Projection) -> LogicalPlan | None:
         required_columns = parent._projection.required_columns()
-        grandchild = child._children()[0]
-        grandchild_output = grandchild.schema()
-        grandchild_ids = grandchild_output.to_id_set()
 
-        can_skip_child = required_columns.to_id_set().issubset(grandchild_ids)
+        parent_projection = parent._projection
+        child_projection = child._projection
+        grandchild = child._children()[0]
+
+        child_mapping = child_projection.input_mapping()
+        can_skip_child = required_columns.issubset(child_mapping.keys())
 
         if can_skip_child:
             logger.debug(f"Folding: {parent} into {child}")
 
             new_exprs = []
-            for e in parent._projection:
-                if isinstance(e, ColumnExpression):
+            for e in parent_projection:
+                if e.is_column():
                     name = e.name()
                     assert name is not None
-                    e = child._projection.get_expression_by_name(name)
+                    e = child_projection.get_expression_by_name(name)
                 else:
-                    for rc in e.required_columns():
-                        name = rc.name()
-                        assert name is not None
-                        e = e._replace_column_with_expression(rc, child._projection.get_expression_by_name(name))
+                    e = copy.deepcopy(e)
+                    to_replace = e.required_columns()
+                    for name in to_replace:
+                        e = e._replace_column_with_expression(col(name), child_projection.get_expression_by_name(name))
                 new_exprs.append(e)
-            return Projection(grandchild, ExpressionList(new_exprs))
+            return Projection(
+                grandchild,
+                ExpressionList(new_exprs),
+                custom_resource_request=resource_request.ResourceRequest.max_resources(
+                    [parent.resource_request(), child.resource_request()]
+                ),
+            )
         else:
             return None
 
@@ -280,9 +315,9 @@ class DropProjections(Rule[LogicalPlan]):
         parent_projection = parent._projection
         child_output = child.schema()
         if (
-            all(isinstance(expr, ColumnExpression) for expr in parent_projection)
+            all(expr.is_column() for expr in parent_projection)
             and len(parent_projection) == len(child_output)
-            and all(p.get_id() == c.get_id() for p, c in zip(parent_projection, child_output))
+            and all(p.name() == c.name for p, c in zip(parent_projection, child_output))
         ):
             return child
         else:
