@@ -6,7 +6,7 @@ import weakref
 from abc import abstractmethod
 from dataclasses import dataclass
 from functools import partial
-from typing import IO, Any, Callable, Generic, List, Sequence, TypeVar, cast
+from typing import IO, Any, Callable, Generic, TypeVar
 from uuid import uuid4
 
 import numpy as np
@@ -17,9 +17,10 @@ from pyarrow import dataset as pada
 from pyarrow import json, parquet
 
 from daft.execution.operators import OperatorEnum
-from daft.expressions import ColID, ColumnExpression, Expression, ExpressionExecutor
+from daft.expressions import Expression, ExpressionExecutor, ExpressionList
 from daft.filesystem import get_filesystem_from_path
-from daft.logical.schema import ExpressionList
+from daft.logical.field import Field
+from daft.logical.schema import Schema
 from daft.runners.blocks import ArrowArrType, ArrowDataBlock, DataBlock, PyListDataBlock
 from daft.types import ExpressionType, PythonExpressionType
 
@@ -48,13 +49,13 @@ class vPartitionSchemaInferenceOptions:
         inference_column_names: Column names to use when performing schema inference
     """
 
-    schema: ExpressionList | None = None
+    schema: Schema | None = None
     inference_column_names: list[str] | None = None
 
     def full_schema_column_names(self) -> list[str] | None:
         """Returns all column names for the schema, or None if not provided."""
         if self.schema is not None:
-            return [expr.name() for expr in self.schema]
+            return self.schema.column_names()
         return self.inference_column_names
 
 
@@ -86,25 +87,8 @@ def _limit_num_rows(buf: IO, num_rows: int) -> IO:
     return sampled_bytes
 
 
-def _get_column_ids(table: pa.Table, schema_options: vPartitionSchemaInferenceOptions) -> list[ColID]:
-    # No schema provided, we allocate column IDs arbitrarily
-    # NOTE: This is very dangerous if performed remotely in Ray and if IDs are used locally, since they may conflict
-    if schema_options.schema is None:
-        return cast(List[ColID], list(range(len(table.column_names))))
-
-    col_ids = []
-    for colname in table.column_names:
-        expr = schema_options.schema.get_expression_by_name(colname)
-        if expr is None:
-            raise ValueError(f"Column {colname} not found in provided schema: {schema_options.schema}")
-        col_ids.append(expr.get_id())
-
-    return col_ids
-
-
 @dataclass(frozen=True)
 class PyListTile:
-    column_id: ColID
     column_name: str
     partition_id: PartID
     block: DataBlock
@@ -139,12 +123,10 @@ class PyListTile:
         if len(to_merge) == 1:
             return to_merge[0]
 
-        column_id = to_merge[0].column_id
         partition_id = to_merge[0].partition_id
         column_name = to_merge[0].column_name
         # first perform sanity check
         for part in to_merge[1:]:
-            assert part.column_id == column_id
             assert not verify_partition_id or part.partition_id == partition_id
             assert part.column_name == column_name
 
@@ -162,20 +144,20 @@ class PartitionMetadata:
 
 @dataclass(frozen=True)
 class vPartition:
-    columns: dict[ColID, PyListTile]
+    columns: dict[str, PyListTile]
     partition_id: PartID
 
     def __post_init__(self) -> None:
         size = None
-        for col_id, tile in self.columns.items():
+        for name, tile in self.columns.items():
             if tile.partition_id != self.partition_id:
                 raise ValueError(f"mismatch of partition id: {tile.partition_id} vs {self.partition_id}")
             if len(tile) != 0 and size is None:
                 size = len(tile)
             if len(tile) != 0 and len(tile) != size:
                 raise ValueError(f"mismatch of tile lengths: {len(tile)} vs {size}")
-            if col_id != tile.column_id:
-                raise ValueError(f"mismatch of column id: {col_id} vs {tile.column_id}")
+            if name != tile.column_name:
+                raise ValueError(f"mismatch of tile name: {name} vs {tile.column_name}")
 
     def __len__(self) -> int:
         if len(self.columns) == 0:
@@ -185,9 +167,9 @@ class vPartition:
     def metadata(self) -> PartitionMetadata:
         return PartitionMetadata(num_rows=len(self))
 
-    def get_unresolved_col_expressions(self) -> ExpressionList:
+    def get_schema(self) -> Schema:
         """Generates column expressions that represent the vPartition's schema"""
-        colexprs = []
+        fields = []
         for _, tile in self.columns.items():
             col_name = tile.column_name
             col_type: ExpressionType
@@ -201,63 +183,51 @@ class vPartition:
                     col_type = PythonExpressionType(py_types.pop())
                 else:
                     col_type = ExpressionType.python_object()
-            colexpr = ColumnExpression(name=col_name, expr_type=col_type)
-            colexprs.append(colexpr)
-        return ExpressionList(colexprs)
+            field = Field(col_name, col_type)
+            fields.append(field)
+        return Schema(fields)
 
     def eval_expression(self, expr: Expression) -> PyListTile:
-        expr_col_id = expr.get_id()
-
         # Avoid recomputing expressions that have been computed before
-        if expr_col_id in self.columns and self.columns[expr_col_id].column_name == expr.name():
-            return self.columns[expr_col_id]
 
         expr_name = expr.name()
 
-        assert expr_col_id is not None
         assert expr_name is not None
 
         required_cols = expr.required_columns()
         required_blocks = {}
-        for c in required_cols:
-            col_id = c.get_id()
-            assert col_id is not None
-            block = self.columns[col_id].block
-            name = c.name()
-            assert name is not None
+        for name in required_cols:
+            block = self.columns[name].block
             required_blocks[name] = block
         exec = ExpressionExecutor()
         result = exec.eval(expr, required_blocks)
-        expr_col_id = expr.get_id()
         expr_name = expr.name()
-        assert expr_col_id is not None
         assert expr_name is not None
-        return PyListTile(column_id=expr_col_id, column_name=expr_name, partition_id=self.partition_id, block=result)
+        return PyListTile(column_name=expr_name, partition_id=self.partition_id, block=result)
 
     def eval_expression_list(self, exprs: ExpressionList) -> vPartition:
         tile_list = [self.eval_expression(e) for e in exprs]
-        new_columns = {t.column_id: t for t in tile_list}
+        new_columns = {t.column_name: t for t in tile_list}
         return vPartition(columns=new_columns, partition_id=self.partition_id)
 
     @classmethod
-    def from_arrow_table(cls, table: pa.Table, column_ids: list[ColID], partition_id: PartID) -> vPartition:
+    def from_arrow_table(cls, table: pa.Table, partition_id: PartID) -> vPartition:
         names = table.column_names
-        assert len(names) == len(column_ids)
         tiles = {}
-        for i, (col_id, name) in enumerate(zip(column_ids, names)):
+        for i, name in enumerate(names):
             arr = table[i]
             block: DataBlock[ArrowArrType] = DataBlock.make_block(arr)
-            tiles[col_id] = PyListTile(column_id=col_id, column_name=name, partition_id=partition_id, block=block)
+            tiles[name] = PyListTile(column_name=name, partition_id=partition_id, block=block)
         return vPartition(columns=tiles, partition_id=partition_id)
 
     @classmethod
-    def from_pydict(cls, data: dict[str, list[Any]], schema: ExpressionList, partition_id: PartID) -> vPartition:
-        column_exprs = schema.to_column_expressions()
+    def from_pydict(cls, data: dict[str, list[Any]], schema: Schema, partition_id: PartID) -> vPartition:
+        fields = schema.fields
         tiles = {}
-        for col_expr in column_exprs:
-            col_id = col_expr.get_id()
-            col_name = col_expr.name()
-            col_type = col_expr.resolved_type()
+        for f in fields.values():
+
+            col_name = f.name
+            col_type = f.dtype
             col_data = data[col_name]
 
             if ExpressionType.is_py(col_type):
@@ -267,7 +237,7 @@ class vPartition:
                     col_data = pa.array(col_data, type=col_type.to_arrow_type())
 
             block = DataBlock.make_block(col_data)
-            tiles[col_id] = PyListTile(column_id=col_id, column_name=col_name, partition_id=partition_id, block=block)
+            tiles[col_name] = PyListTile(column_name=col_name, partition_id=partition_id, block=block)
         return vPartition(columns=tiles, partition_id=partition_id)
 
     @classmethod
@@ -319,8 +289,7 @@ class vPartition:
                 convert_options=csv.ConvertOptions(include_columns=read_options.column_names),
             )
 
-        column_ids = _get_column_ids(table, schema_options)
-        return vPartition.from_arrow_table(table, column_ids=column_ids, partition_id=partition_id)
+        return vPartition.from_arrow_table(table, partition_id=partition_id)
 
     @classmethod
     def from_json(
@@ -347,8 +316,7 @@ class vPartition:
         if read_options.column_names is not None:
             table = table.select(read_options.column_names)
 
-        column_ids = _get_column_ids(table, schema_options)
-        return vPartition.from_arrow_table(table, column_ids=column_ids, partition_id=partition_id)
+        return vPartition.from_arrow_table(table, partition_id=partition_id)
 
     @classmethod
     def from_parquet(
@@ -384,22 +352,28 @@ class vPartition:
                 if read_options.num_rows is not None:
                     table = table.slice(length=read_options.num_rows)
 
-        column_ids = _get_column_ids(table, schema_options)
-        return vPartition.from_arrow_table(table, column_ids=column_ids, partition_id=partition_id)
+        return vPartition.from_arrow_table(table, partition_id=partition_id)
 
-    def to_pydict(self) -> dict[str, Sequence]:
+    def to_pydict(self) -> dict[str, list[Any]]:
         output_schema = [(tile.column_name, id) for id, tile in self.columns.items()]
-        return {name: self.columns[id].block.data for name, id in output_schema}
 
-    def to_pandas(self, schema: ExpressionList | None = None) -> pd.DataFrame:
-        if schema is not None:
-            output_schema = [(expr.name(), expr.get_id()) for expr in schema]
-        else:
-            output_schema = [(tile.column_name, id) for id, tile in self.columns.items()]
-
-        data = {}
+        results = {}
         for name, id in output_schema:
             block = self.columns[id].block
+            data = [block.data for _ in range(len(self))] if block.is_scalar() else list(block.iter_py())
+            results[name] = data
+
+        return results
+
+    def to_pandas(self, schema: Schema | None = None) -> pd.DataFrame:
+        if schema is not None:
+            output_schema = [f.name for f in schema.fields.values()]
+        else:
+            output_schema = [tile for tile in self.columns.keys()]
+
+        data = {}
+        for name in output_schema:
+            block = self.columns[name].block
             # PyListDataBlocks contain Python objects
             if isinstance(block, PyListDataBlock) and block.is_scalar():
                 data[name] = pd.Series([block.data for _ in range(len(self))])
@@ -414,7 +388,7 @@ class vPartition:
         return pd.DataFrame(data)
 
     def for_each_column_block(self, func: Callable[[DataBlock], DataBlock]) -> vPartition:
-        return dataclasses.replace(self, columns={col_id: col.apply(func) for col_id, col in self.columns.items()})
+        return dataclasses.replace(self, columns={col_name: col.apply(func) for col_name, col in self.columns.items()})
 
     def head(self, num: int) -> vPartition:
         # TODO make optimization for when num=0
@@ -450,10 +424,10 @@ class vPartition:
 
     def search_sorted(self, keys: vPartition, input_reversed: list[bool] | None = None) -> DataBlock:
         assert self.columns.keys() == keys.columns.keys()
-        col_ids = list(self.columns.keys())
+        col_names = list(self.columns.keys())
         idx = DataBlock.search_sorted(
-            [self.columns[k].block for k in col_ids],
-            [keys.columns[k].block for k in col_ids],
+            [self.columns[k].block for k in col_names],
+            [keys.columns[k].block for k in col_names],
             input_reversed=input_reversed,
         )
         return idx
@@ -466,8 +440,8 @@ class vPartition:
         ops = [op for _, op in to_agg]
         if group_by is None:
             agged = {}
-            for op, (col_id, tile) in zip(ops, evaled_expressions.columns.items()):
-                agged[col_id] = tile.apply(func=partial(tile.block.__class__.agg, op=op))
+            for op, (col_name, tile) in zip(ops, evaled_expressions.columns.items()):
+                agged[col_name] = tile.apply(func=partial(tile.block.__class__.agg, op=op))
             return vPartition(partition_id=self.partition_id, columns=agged)
         else:
             grouped_blocked = self.eval_expression_list(group_by)
@@ -479,11 +453,11 @@ class vPartition:
             )
             new_columns = {}
 
-            for block, (col_id, tile) in zip(gcols, grouped_blocked.columns.items()):
-                new_columns[col_id] = dataclasses.replace(tile, block=block)
+            for block, (col_name, tile) in zip(gcols, grouped_blocked.columns.items()):
+                new_columns[col_name] = dataclasses.replace(tile, block=block)
 
-            for block, (col_id, tile) in zip(acols, evaled_expressions.columns.items()):
-                new_columns[col_id] = dataclasses.replace(tile, block=block)
+            for block, (col_name, tile) in zip(acols, evaled_expressions.columns.items()):
+                new_columns[col_name] = dataclasses.replace(tile, block=block)
             return vPartition(partition_id=self.partition_id, columns=new_columns)
 
     def split_by_hash(self, exprs: ExpressionList, num_partitions: int) -> list[vPartition]:
@@ -501,14 +475,14 @@ class vPartition:
 
     def split_by_index(self, num_partitions: int, target_partition_indices: DataBlock) -> list[vPartition]:
         assert len(target_partition_indices) == len(self)
-        new_partition_to_columns: list[dict[ColID, PyListTile]] = [{} for _ in range(num_partitions)]
+        new_partition_to_columns: list[dict[str, PyListTile]] = [{} for _ in range(num_partitions)]
         argsort_targets = DataBlock.argsort([target_partition_indices])
         sorted_targets = target_partition_indices.take(argsort_targets)
         sorted_targets_np = sorted_targets.to_numpy()
         pivots = np.where(np.diff(sorted_targets_np, prepend=np.nan))[0]
         target_partitions = sorted_targets_np[pivots]
 
-        for col_id, tile in self.columns.items():
+        for name, tile in self.columns.items():
             new_tiles = tile.split_by_index(
                 num_partitions=num_partitions,
                 pivots=pivots,
@@ -516,7 +490,7 @@ class vPartition:
                 argsorted_target_partition_indices=argsort_targets,
             )
             for part_id, nt in enumerate(new_tiles):
-                new_partition_to_columns[part_id][col_id] = nt
+                new_partition_to_columns[part_id][name] = nt
 
         return [vPartition(partition_id=i, columns=columns) for i, columns in enumerate(new_partition_to_columns)]
 
@@ -533,13 +507,13 @@ class vPartition:
         partition_to_explode = self.eval_expression_list(columns)
         exploded_col_names = {tile.column_name for tile in partition_to_explode.columns.values()}
         partition_to_repeat = vPartition(
-            {cid: tile for cid, tile in self.columns.items() if tile.column_name not in exploded_col_names},
+            {name: tile for name, tile in self.columns.items() if name not in exploded_col_names},
             partition_id=self.partition_id,
         )
 
         exploded_cols = {}
         found_list_lengths = None
-        for col_id, tile in partition_to_explode.columns.items():
+        for name, tile in partition_to_explode.columns.items():
             exploded_block, list_lengths = tile.block.list_explode()
 
             # Ensure that each row has the same length as other rows
@@ -552,9 +526,8 @@ class vPartition:
                         "but found row(s) with mismatched lengths"
                     )
 
-            exploded_cols[col_id] = PyListTile(
-                column_id=tile.column_id,
-                column_name=tile.column_name,
+            exploded_cols[name] = PyListTile(
+                column_name=name,
                 partition_id=tile.partition_id,
                 block=exploded_block,
             )
@@ -585,7 +558,7 @@ class vPartition:
         right: vPartition,
         left_on: ExpressionList,
         right_on: ExpressionList,
-        output_schema: ExpressionList,
+        output_projection: ExpressionList,
         how: str = "inner",
     ) -> vPartition:
         assert how == "inner"
@@ -593,8 +566,8 @@ class vPartition:
         left_key_ids = list(left_key_part.columns.keys())
 
         right_key_part = right.eval_expression_list(right_on)
-        left_key_list = [left_key_part.columns[le.get_id()].block for le in left_on]
-        right_key_list = [right_key_part.columns[re.get_id()].block for re in right_on]
+        left_key_list = [left_key_part.columns[le.name()].block for le in left_on]
+        right_key_list = [right_key_part.columns[re.name()].block for re in right_on]
 
         left_nonjoin_ids = [i for i in self.columns.keys() if i not in left_key_part.columns]
         right_nonjoin_ids = [i for i in right.columns.keys() if i not in right_key_part.columns]
@@ -624,7 +597,7 @@ class vPartition:
         assert joined_block_idx == len(result_keys)
 
         output = vPartition(columns=result_columns, partition_id=self.partition_id)
-        return output.eval_expression_list(output_schema)
+        return output.eval_expression_list(output_projection)
 
     def _to_file(
         self,
@@ -633,20 +606,18 @@ class vPartition:
         partition_cols: ExpressionList | None = None,
         compression: str | None = None,
     ) -> list[str]:
-        keys = [col_id for col_id in self.columns.keys()]
+        keys = [col_name for col_name in self.columns.keys()]
         names = [self.columns[k].column_name for k in keys]
         data = [self.columns[k].block.to_arrow() for k in keys]
         arrow_table = pa.table(data, names=names)
         partition_col_names = []
         if partition_cols is not None:
             for col in partition_cols:
-                assert isinstance(
-                    col, ColumnExpression
-                ), "we can only support ColumnExpressions for partitioning parquet"
-                col_id = col.get_id()
-                assert col_id is not None
-                assert col_id in keys
-                partition_col_names.append(self.columns[col_id].column_name)
+                assert col.is_column(), "we can only support Column Expressions for partitioning parquet"
+                col_name = col.name()
+                assert col_name is not None
+                assert col_name in keys
+                partition_col_names.append(col_name)
 
         visited_paths = []
 
@@ -694,16 +665,16 @@ class vPartition:
             return to_merge[0]
 
         pid = to_merge[0].partition_id
-        col_ids = set(to_merge[0].columns.keys())
+        col_names = set(to_merge[0].columns.keys())
         # first perform sanity check
         for part in to_merge[1:]:
             assert not verify_partition_id or part.partition_id == pid
-            assert set(part.columns.keys()) == col_ids
+            assert set(part.columns.keys()) == col_names
 
         new_columns = {}
-        for col_id in to_merge[0].columns.keys():
-            new_columns[col_id] = PyListTile.merge_tiles(
-                [vp.columns[col_id] for vp in to_merge], verify_partition_id=verify_partition_id
+        for col_name in to_merge[0].columns.keys():
+            new_columns[col_name] = PyListTile.merge_tiles(
+                [vp.columns[col_name] for vp in to_merge], verify_partition_id=verify_partition_id
             )
         return dataclasses.replace(to_merge[0], columns=new_columns)
 
@@ -715,12 +686,12 @@ class PartitionSet(Generic[PartitionT]):
     def _get_merged_vpartition(self) -> vPartition:
         raise NotImplementedError()
 
-    def to_pydict(self) -> dict[str, Sequence]:
+    def to_pydict(self) -> dict[str, list[Any]]:
         """Retrieves all the data in a PartitionSet as a Python dictionary. Values are the raw data from each Block."""
         merged_partition = self._get_merged_vpartition()
         return merged_partition.to_pydict()
 
-    def to_pandas(self, schema: ExpressionList | None = None) -> pd.DataFrame:
+    def to_pandas(self, schema: Schema | None = None) -> pd.DataFrame:
         merged_partition = self._get_merged_vpartition()
         return merged_partition.to_pandas(schema=schema)
 
@@ -814,20 +785,56 @@ class PartitionSetCache:
 class PartitionSetFactory(Generic[PartitionT]):
     """Factory class for creating PartitionSets."""
 
-    FILEPATH_COLUMN_NAME = "filepath"
+    FS_LISTING_PATH_COLUMN_NAME = "path"
+    FS_LISTING_SIZE_COLUMN_NAME = "size"
+    FS_LISTING_TYPE_COLUMN_NAME = "type"
+
+    def _get_listing_paths_schema(self) -> Schema:
+        """Construct the schema for a DataFrame of path listing"""
+        return Schema(
+            [
+                Field(self.FS_LISTING_PATH_COLUMN_NAME, ExpressionType.string()),
+            ]
+        )
+
+    def _get_listing_paths_details_schema(self) -> Schema:
+        """Construct the schema for a DataFrame of detailed path listing"""
+        return Schema(
+            [
+                Field(self.FS_LISTING_PATH_COLUMN_NAME, ExpressionType.string()),
+                Field(self.FS_LISTING_SIZE_COLUMN_NAME, ExpressionType.integer()),
+                Field(self.FS_LISTING_TYPE_COLUMN_NAME, ExpressionType.string()),
+            ]
+        )
 
     @abstractmethod
-    def glob_filepaths(
+    def glob_paths(
         self,
         source_path: str,
-    ) -> tuple[PartitionSet[PartitionT], ExpressionList]:
-        """Globs the specified filepath to construct a PartitionSet of file metadata
+    ) -> tuple[PartitionSet[PartitionT], Schema]:
+        """Globs the specified filepath to construct a PartitionSet of file or dir paths
 
         Args:
             source_path (str): path to glob
 
         Returns:
-            PartitionSet[PartitionT]: PartitionSet containing the files' metadata
+            PartitionSet[PartitionT]: PartitionSet containing the listings' paths
+            ExpressionList: Schema of the PartitionSet that was constructed
+        """
+        raise NotImplementedError()
+
+    @abstractmethod
+    def glob_paths_details(
+        self,
+        source_path: str,
+    ) -> tuple[PartitionSet[PartitionT], Schema]:
+        """Globs the specified filepath to construct a PartitionSet of file and dir metadata
+
+        Args:
+            source_path (str): path to glob
+
+        Returns:
+            PartitionSet[PartitionT]: PartitionSet containing the listings' metadata
             ExpressionList: Schema of the PartitionSet that was constructed
         """
         raise NotImplementedError()
