@@ -5,25 +5,33 @@ from datetime import datetime
 from typing import TYPE_CHECKING, Any, Callable, ClassVar, Iterator
 
 import pyarrow as pa
-import ray
 from loguru import logger
 
+try:
+    import ray
+except ImportError:
+    logger.error(
+        f"Error when importing Ray. Please ensure that getdaft was installed with the Ray extras tag: getdaft[ray] (https://getdaft.io/docs/learn/install.html)"
+    )
+    raise
+
 from daft.execution import physical_plan_factory
 from daft.execution.execution_plan import ExecutionPlan
 from daft.execution.execution_step import (
+    FanoutInstruction,
     Instruction,
     MaterializationRequest,
     MaterializationRequestBase,
     MaterializationRequestMulti,
     MaterializationResult,
+    ReduceInstruction,
 )
 from daft.execution.logical_op_runners import (
     LogicalGlobalOpRunner,
     LogicalPartitionOpRunner,
     ReduceType,
 )
-from daft.expressions import ColumnExpression
-from daft.filesystem import glob_path
+from daft.filesystem import glob_path, glob_path_with_stats
 from daft.internal.rule_runner import FixedPointPolicy, Once, RuleBatch, RuleRunner
 from daft.logical import logical_plan
 from daft.logical.optimizer import (
@@ -35,7 +43,6 @@ from daft.logical.optimizer import (
     PushDownLimit,
     PushDownPredicates,
 )
-from daft.logical.schema import ExpressionList
 from daft.resource_request import ResourceRequest
 from daft.runners.blocks import ArrowDataBlock, zip_blocks_as_py
 from daft.runners.partitioning import (
@@ -57,7 +64,6 @@ from daft.runners.shuffle_ops import (
     Shuffler,
     SortOp,
 )
-from daft.types import ExpressionType
 
 if TYPE_CHECKING:
     from ray.data.block import Block as RayDatasetBlock
@@ -69,23 +75,52 @@ try:
 except ImportError:
     _RAY_FROM_ARROW_REFS_AVAILABLE = False
 
+from daft.logical.schema import Schema
+
 
 @ray.remote
-def _glob_path_into_vpartitions(path: str, schema: ExpressionList) -> list[tuple[PartID, vPartition]]:
+def _glob_path_into_vpartitions(path: str, schema: Schema) -> list[tuple[PartID, vPartition]]:
     assert len(schema) == 1
-    filepath_expr = list(schema)[0]
-    filepaths = glob_path(path)
+    listing_path_name = "path"
+
+    listing_paths = glob_path(path)
+    if len(listing_paths) == 0:
+        raise FileNotFoundError(f"No files found at {path}")
 
     # Hardcoded to 1 filepath per partition
     partition_refs = []
-    for i, filepath in enumerate(filepaths):
-        partition = vPartition.from_pydict({filepath_expr.name(): [filepath]}, schema=schema, partition_id=i)
+    for i, path in enumerate(listing_paths):
+        partition = vPartition.from_pydict({listing_path_name: [path]}, schema=schema, partition_id=i)
         partition_ref = ray.put(partition)
         partition_refs.append((i, partition_ref))
 
     return partition_refs
 
 
+@ray.remote
+def _glob_path_into_details_vpartitions(path: str, schema: Schema) -> list[tuple[PartID, vPartition]]:
+    assert len(schema) == 3
+    listing_path_name, listing_size_name, listing_type_name = ["path", "size", "type"]
+    listing_infos = glob_path_with_stats(path)
+    if len(listing_infos) == 0:
+        raise FileNotFoundError(f"No files found at {path}")
+
+    # Hardcoded to 1 partition
+    partition = vPartition.from_pydict(
+        {
+            listing_path_name: [file_info.path for file_info in listing_infos],
+            listing_size_name: [file_info.size for file_info in listing_infos],
+            listing_type_name: [file_info.type for file_info in listing_infos],
+        },
+        schema=schema,
+        partition_id=0,
+    )
+    partition_ref = ray.put(partition)
+    partition_refs = [(0, partition_ref)]
+
+    return partition_refs
+
+
 @ray.remote
 def _make_ray_block_from_vpartition(partition: vPartition) -> RayDatasetBlock:
     daft_blocks = {tile.column_name: tile.block for _, tile in partition.columns.items()}
@@ -99,6 +134,11 @@ def _make_ray_block_from_vpartition(partition: vPartition) -> RayDatasetBlock:
     return [dict(zip(colnames, row_tuple)) for row_tuple in zip_blocks_as_py(*blocks)]
 
 
+@ray.remote
+def remote_len_partition(p: vPartition) -> int:
+    return len(p)
+
+
 @dataclass
 class RayPartitionSet(PartitionSet[ray.ObjectRef]):
     _partitions: dict[PartID, ray.ObjectRef]
@@ -142,11 +182,7 @@ class RayPartitionSet(PartitionSet[ray.ObjectRef]):
     def len_of_partitions(self) -> list[int]:
         partition_ids = sorted(list(self._partitions.keys()))
 
-        @ray.remote
-        def remote_len(p: vPartition) -> int:
-            return len(p)
-
-        result: list[int] = ray.get([remote_len.remote(self._partitions[pid]) for pid in partition_ids])
+        result: list[int] = ray.get([remote_len_partition.remote(self._partitions[pid]) for pid in partition_ids])
         return result
 
     def num_partitions(self) -> int:
@@ -157,20 +193,27 @@ class RayPartitionSet(PartitionSet[ray.ObjectRef]):
 
 
 class RayPartitionSetFactory(PartitionSetFactory[ray.ObjectRef]):
-    def glob_filepaths(
+    def glob_paths(
         self,
         source_path: str,
-    ) -> tuple[RayPartitionSet, ExpressionList]:
-        schema = ExpressionList([ColumnExpression(self.FILEPATH_COLUMN_NAME, ExpressionType.string())]).resolve()
+    ) -> tuple[RayPartitionSet, Schema]:
+        schema = self._get_listing_paths_schema()
         partition_refs = ray.get(_glob_path_into_vpartitions.remote(source_path, schema))
         return RayPartitionSet({part_id: part for part_id, part in partition_refs}), schema
 
+    def glob_paths_details(
+        self,
+        source_path: str,
+    ) -> tuple[RayPartitionSet, Schema]:
+        schema = self._get_listing_paths_details_schema()
+        partition_refs = ray.get(_glob_path_into_details_vpartitions.remote(source_path, schema))
+        return RayPartitionSet({part_id: part for part_id, part in partition_refs}), schema
+
 
 class RayRunnerSimpleShuffler(Shuffler):
     def run(self, input: PartitionSet, num_target_partitions: int) -> PartitionSet:
         map_args = self._map_args if self._map_args is not None else {}
         reduce_args = self._reduce_args if self._reduce_args is not None else {}
-        ray_expr_eval_task_options = _get_ray_task_options(self._expr_eval_resource_request)
 
         source_partitions = input.num_partitions()
 
@@ -190,10 +233,7 @@ class RayRunnerSimpleShuffler(Shuffler):
             else:
                 return output_list
 
-        map_results = [
-            map_wrapper.options(**ray_expr_eval_task_options).remote(input=input.get_partition(i))
-            for i in range(source_partitions)
-        ]
+        map_results = [map_wrapper.remote(input=input.get_partition(i)) for i in range(source_partitions)]
 
         if num_target_partitions == 1:
             ray.wait(map_results)
@@ -206,9 +246,7 @@ class RayRunnerSimpleShuffler(Shuffler):
                 map_subset = map_results
             else:
                 map_subset = [map_results[i][t] for i in range(source_partitions)]
-            # NOTE: not all reduce ops actually require ray_expr_eval_task_options. This is an area for
-            # potential improvement for repartitioning operations which only require the task options for mapping
-            reduced_part = reduce_wrapper.options(**ray_expr_eval_task_options).remote(*map_subset)
+            reduced_part = reduce_wrapper.remote(*map_subset)
             reduced_results.append(reduced_part)
 
         return RayPartitionSet({i: part for i, part in enumerate(reduced_results)})
@@ -283,10 +321,8 @@ class RayLogicalGlobalOpRunner(LogicalGlobalOpRunner):
         SortOp: RayRunnerSortOp,
     }
 
-    def map_partitions(
-        self, pset: PartitionSet, func: Callable[[vPartition], vPartition], resource_request: ResourceRequest
-    ) -> PartitionSet:
-        remote_func = ray.remote(func).options(**_get_ray_task_options(resource_request))
+    def map_partitions(self, pset: PartitionSet, func: Callable[[vPartition], vPartition]) -> PartitionSet:
+        remote_func = ray.remote(func)
         return RayPartitionSet({i: remote_func.remote(pset.get_partition(i)) for i in range(pset.num_partitions())})
 
     def reduce_partitions(self, pset: PartitionSet, func: Callable[[list[vPartition]], ReduceType]) -> ReduceType:
@@ -342,7 +378,10 @@ class RayRunner(Runner):
         exec_plan = ExecutionPlan.plan_from_logical(plan)
         result_partition_set: PartitionSet
         partition_intermediate_results: dict[int, PartitionSet] = {}
-        with profiler("profile_RayRunner.run_{datetime.now().isoformat()}.json"):
+        profile_filename = (
+            f"profile_RayRunner.run()_{datetime.replace(datetime.now(), second=0, microsecond=0).isoformat()[:-3]}.json"
+        )
+        with profiler(profile_filename):
             for exec_op in exec_plan.execution_ops:
 
                 data_deps = exec_op.data_deps
@@ -369,7 +408,6 @@ class RayRunner(Runner):
             return pset_entry
 
 
-@ray.remote
 def build_partitions(instruction_stack: list[Instruction], *inputs: vPartition) -> vPartition | list[vPartition]:
     partitions = list(inputs)
     for instruction in instruction_stack:
@@ -378,40 +416,77 @@ def build_partitions(instruction_stack: list[Instruction], *inputs: vPartition)
     return partitions if len(partitions) > 1 else partitions[0]
 
 
+# Give the same function different names to aid in profiling data distribution.
+
+
+@ray.remote
+def pipeline_build(instruction_stack: list[Instruction], *inputs: vPartition) -> vPartition | list[vPartition]:
+    return build_partitions(instruction_stack, *inputs)
+
+
+@ray.remote
+def fanout_build(instruction_stack: list[Instruction], *inputs: vPartition) -> vPartition | list[vPartition]:
+    return build_partitions(instruction_stack, *inputs)
+
+
+@ray.remote(scheduling_strategy="SPREAD")
+def reduce_build(instruction_stack: list[Instruction], *inputs: vPartition) -> vPartition | list[vPartition]:
+    return build_partitions(instruction_stack, *inputs)
+
+
+@ray.remote(scheduling_strategy="SPREAD")
+def reduce_fanout_build(instruction_stack: list[Instruction], *inputs: vPartition) -> vPartition | list[vPartition]:
+    return build_partitions(instruction_stack, *inputs)
+
+
 @ray.remote
 def get_meta(partition: vPartition) -> PartitionMetadata:
     return partition.metadata()
 
 
-class DynamicRayRunner(RayRunner):
-    def run(self, plan: logical_plan.LogicalPlan) -> PartitionCacheEntry:
+@ray.remote
+def remote_run_plan(
+    plan: logical_plan.LogicalPlan,
+    psets: dict[str, ray.ObjectRef],
+    max_tasks_per_core: int,
+    max_refs_per_core: int,
+) -> list[ray.ObjectRef]:
 
-        plan = self.optimize(plan)
+    from loguru import logger
 
-        phys_plan: Iterator[
-            None | MaterializationRequestBase[vPartition]
-        ] = physical_plan_factory.get_materializing_physical_plan(plan)
+    phys_plan: Iterator[
+        None | MaterializationRequestBase[vPartition]
+    ] = physical_plan_factory.get_materializing_physical_plan(plan, psets)
 
-        # Note: For autoscaling clusters, we will probably want to query this dynamically.
-        # Keep in mind this call takes about 0.3ms.
-        cores = int(ray.cluster_resources()["CPU"])
-        constructions_to_dispatch = []
-        inflight_constructions: dict[str, MaterializationRequestBase[ray.ObjectRef]] = dict()
-        inflight_ref_to_construction: dict[ray.ObjectRef, str] = dict()
+    # Note: For autoscaling clusters, we will probably want to query cores dynamically.
+    # Keep in mind this call takes about 0.3ms.
+    cores = int(ray.cluster_resources()["CPU"])
 
-        start = datetime.now()
-        result_pset = RayPartitionSet({})
-        with profiler("profile_DynamicRayRunner.run_{datetime.now().isoformat()}.json"):
+    inflight_tasks: dict[str, MaterializationRequestBase[ray.ObjectRef]] = dict()
+    inflight_ref_to_task: dict[ray.ObjectRef, str] = dict()
 
-            try:
+    result_partitions = None
+    start = datetime.now()
+    profile_filename = (
+        f"profile_DynamicRayRunner.run()_"
+        f"{datetime.replace(datetime.now(), second=0, microsecond=0).isoformat()[:-3]}.json"
+    )
+    with profiler(profile_filename):
+        while True:
 
-                while True:
-                    # Dispatch tasks while cores are available.
-                    cores_available = cores - len(inflight_constructions)
-                    for i in range(cores_available):
+            while (
+                len(inflight_tasks) < max_tasks_per_core * cores
+                and len(inflight_ref_to_task) < max_refs_per_core * cores
+            ):
+
+                # Get the next batch of tasks to dispatch.
+                tasks_to_dispatch = []
+                try:
+                    for _ in range(cores):
 
                         next_step = next(phys_plan)
 
+                        # If this task is a no-op, just run it locally immediately.
                         while next_step is not None and len(next_step.instructions) == 0:
                             assert isinstance(next_step, MaterializationRequest)
                             [partition] = next_step.inputs
@@ -421,97 +496,113 @@ class DynamicRayRunner(RayRunner):
                         if next_step is None:
                             break
 
-                        constructions_to_dispatch.append(next_step)
+                        tasks_to_dispatch.append(next_step)
 
-                    dispatch = datetime.now()
-                    logger.debug(
-                        f"{(dispatch - start).total_seconds()}s: DynamicRayRunner dispatched batch of {len(constructions_to_dispatch)} tasks."
-                    )
-                    for construction in constructions_to_dispatch:
-                        results = self._build_partitions(construction)
-                        logger.debug(f"{construction} -> {results}")
-                        inflight_constructions[construction.id()] = construction
-                        for result in results:
-                            inflight_ref_to_construction[result] = construction.id()
+                except StopIteration as e:
+                    result_partitions = e.value
 
-                    constructions_to_dispatch.clear()
+                # Dispatch the batch of tasks.
+                logger.debug(
+                    f"{(datetime.now() - start).total_seconds()}s: DynamicRayRunner dispatching {len(tasks_to_dispatch)} tasks:"
+                )
+                for task in tasks_to_dispatch:
+                    results = _build_partitions(task)
+                    logger.debug(f"{task} -> {results}")
+                    inflight_tasks[task.id()] = task
+                    for result in results:
+                        inflight_ref_to_task[result] = task.id()
 
-                    # All tasks dispatched. Await a result
-                    # while inflight_ref_to_construction:
-                    dispatch = datetime.now()
-                    [ready], _ = ray.wait(list(inflight_ref_to_construction.keys()), fetch_local=False)
-                    cons_id = inflight_ref_to_construction[ready]
-                    logger.debug(f"+{(datetime.now() - dispatch).total_seconds()}s to await a result from {cons_id}")
+                # Exit if the plan is complete and the result references are available.
+                if result_partitions is not None:
+                    return result_partitions
 
-                    # Flush the entire task associated with the result
-
-                    cons = inflight_constructions[cons_id]
-                    if isinstance(cons, MaterializationRequest):
-                        del inflight_ref_to_construction[ready]
-                    elif isinstance(cons, MaterializationRequestMulti):
-
-                        assert cons.results is not None
-                        for result in cons.results:
-                            del inflight_ref_to_construction[result.partition()]
-
-                    del inflight_constructions[cons_id]
-
-            except StopIteration as e:
-                for i, partition in enumerate(result_partitions := e.value):
-                    result_pset.set_partition(i, partition)
-
-            for construction in constructions_to_dispatch:
-                results = self._build_partitions(construction)
-                inflight_constructions[construction.id()] = construction
-                for result in results:
-                    inflight_ref_to_construction[result] = construction.id()
-
-            constructions_to_dispatch.clear()
-
-            while inflight_ref_to_construction:
+            # Await a batch of tasks.
+            for i in range(min(cores, len(inflight_tasks))):
                 dispatch = datetime.now()
-                [ready], _ = ray.wait(list(inflight_ref_to_construction.keys()), fetch_local=False)
-                cons_id = inflight_ref_to_construction[ready]
-                logger.debug(f"+{(datetime.now() - dispatch).total_seconds()}s to await a result of {cons_id}")
+                [ready], _ = ray.wait(list(inflight_ref_to_task.keys()), fetch_local=False)
+                task_id = inflight_ref_to_task[ready]
+                logger.debug(f"+{(datetime.now() - dispatch).total_seconds()}s to await a result from {task_id}")
 
-                # Flush the entire task associated with the result
-                cons = inflight_constructions[cons_id]
-                if isinstance(cons, MaterializationRequest):
-                    del inflight_ref_to_construction[ready]
-                elif isinstance(cons, MaterializationRequestMulti):
+                # Mark the entire task associated with the result as done.
+                task = inflight_tasks[task_id]
+                if isinstance(task, MaterializationRequest):
+                    del inflight_ref_to_task[ready]
+                elif isinstance(task, MaterializationRequestMulti):
+                    assert task.results is not None
+                    for result in task.results:
+                        del inflight_ref_to_task[result.partition()]
 
-                    assert cons.results is not None
-                    for result in cons.results:
-                        del inflight_ref_to_construction[result.partition()]
+                del inflight_tasks[task_id]
 
-                del inflight_constructions[cons_id]
+
+def _build_partitions(task: MaterializationRequestBase[ray.ObjectRef]) -> list[ray.ObjectRef]:
+    """Run a MaterializationRequest and return the resulting list of partitions."""
+    ray_options: dict[str, Any] = {
+        "num_returns": task.num_results,
+    }
+
+    if task.resource_request is not None:
+        ray_options = {**ray_options, **_get_ray_task_options(task.resource_request)}
+
+    if isinstance(task.instructions[0], ReduceInstruction):
+        build_remote = reduce_fanout_build if isinstance(task.instructions[-1], FanoutInstruction) else reduce_build
+    else:
+        build_remote = fanout_build if isinstance(task.instructions[-1], FanoutInstruction) else pipeline_build
+
+    build_remote = build_remote.options(**ray_options)
+    partitions = build_remote.remote(task.instructions, *task.inputs)
+    # Handle ray bug that ignores list interpretation when num_returns=1
+    if task.num_results == 1:
+        partitions = [partitions]
+
+    if isinstance(task, MaterializationRequestMulti):
+        task.results = [RayMaterializationResult(partition) for partition in partitions]
+    elif isinstance(task, MaterializationRequest):
+        [partition] = partitions
+        task.result = RayMaterializationResult(partition)
+    else:
+        raise TypeError(f"Could not type match input {task}")
+
+    return partitions
+
+
+class DynamicRayRunner(RayRunner):
+    def __init__(
+        self,
+        address: str | None,
+        max_tasks_per_core: int | None,
+        max_refs_per_core: int | None,
+    ) -> None:
+        super().__init__(address=address)
+        self.max_tasks_per_core = max_tasks_per_core if max_tasks_per_core is not None else 4
+        self.max_refs_per_core = max_refs_per_core if max_refs_per_core is not None else 10000
+
+    def run(self, plan: logical_plan.LogicalPlan) -> PartitionCacheEntry:
+        result_pset = RayPartitionSet({})
+
+        plan = self.optimize(plan)
+
+        psets = {
+            key: entry.value.values()
+            for key, entry in self._part_set_cache._uuid_to_partition_set.items()
+            if entry.value is not None
+        }
+        partitions = ray.get(
+            remote_run_plan.remote(
+                plan=plan,
+                psets=psets,
+                max_tasks_per_core=self.max_tasks_per_core,
+                max_refs_per_core=self.max_refs_per_core,
+            )
+        )
+
+        for i, partition in enumerate(partitions):
+            result_pset.set_partition(i, partition)
 
         pset_entry = self._part_set_cache.put_partition_set(result_pset)
+
         return pset_entry
 
-    def _build_partitions(self, partspec: MaterializationRequestBase[ray.ObjectRef]) -> list[ray.ObjectRef]:
-        ray_options: dict[str, Any] = {
-            "num_returns": partspec.num_results,
-        }
-        if len(partspec.inputs) > 1:
-            ray_options["scheduling_strategy"] = "SPREAD"
-
-        construct_remote = build_partitions.options(**ray_options)
-        partitions = construct_remote.remote(partspec.instructions, *partspec.inputs)
-        # Handle ray bug that ignores list interpretation when num_returns=1
-        if partspec.num_results == 1:
-            partitions = [partitions]
-
-        if isinstance(partspec, MaterializationRequestMulti):
-            partspec.results = [RayMaterializationResult(partition) for partition in partitions]
-        elif isinstance(partspec, MaterializationRequest):
-            [partition] = partitions
-            partspec.result = RayMaterializationResult(partition)
-        else:
-            raise TypeError(f"Could not type match input {partspec}")
-
-        return partitions
-
 
 @dataclass(frozen=True)
 class RayMaterializationResult(MaterializationResult[ray.ObjectRef]):
